@@ -19,6 +19,7 @@ const errors = [
   error.InvalidElementStateError,
   error.InvalidSelectorError,
   error.InvalidSessionIDError,
+  error.InvalidWebExtensionError,
   error.JavaScriptError,
   error.MoveTargetOutOfBoundsError,
   error.NoSuchAlertError,
@@ -360,6 +361,14 @@ add_task(function test_InvalidSessionIDError() {
   ok(err instanceof error.WebDriverError);
 });
 
+add_task(function test_InvalidWebExtensionError() {
+  let err = new error.InvalidWebExtensionError("foo");
+  equal("InvalidWebExtensionError", err.name);
+  equal("foo", err.message);
+  equal("invalid web extension", err.status);
+  ok(err instanceof error.WebDriverError);
+});
+
 add_task(function test_JavaScriptError() {
   let err = new error.JavaScriptError("foo");
   equal("JavaScriptError", err.name);
@@ -373,6 +382,8 @@ add_task(function test_JavaScriptError() {
   let inheritedErr = new error.JavaScriptError(superErr);
   equal("RangeError: foo", inheritedErr.message);
   equal(superErr.stack, inheritedErr.stack);
+  equal(superErr.lineNumber, inheritedErr.lineNumber);
+  equal(superErr.columnNumber, inheritedErr.columnNumber);
 });
 
 add_task(function test_MoveTargetOutOfBoundsError() {
@@ -463,6 +474,14 @@ add_task(function test_NoSuchUserContextError() {
   ok(err instanceof error.WebDriverError);
 });
 
+add_task(function test_NoSuchWebExtensionError() {
+  let err = new error.NoSuchWebExtensionError("foo");
+  equal("NoSuchWebExtensionError", err.name);
+  equal("foo", err.message);
+  equal("no such web extension", err.status);
+  ok(err instanceof error.WebDriverError);
+});
+
 add_task(function test_NoSuchWindowError() {
   let err = new error.NoSuchWindowError("foo");
   equal("NoSuchWindowError", err.name);
